#!/bin/bash

# notify_bday.sh														
#																
# This script notifies you of any upcoming birthday or any event
# in general. Events are maintained in bday.dat file. The script 
# has been tested on Bash and Korn shells, and it doesn't run
# on Bourne shell.
#
# Copyright (C) 2010 Barun Kumar Saha
#
# This module is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This module is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.


# *** Change this path as required ***
PROG_PATH=$HOME/bday_notify

### Sub functions ###

#
# Name:		display_usage
# Inputs:	None
# Output:	Displays the different options available for use
#
function display_usage {
	echo "Usage: $0 [OPTIONS]"
	echo ""
	echo -ne "\t-d N, --days N"
	echo -e	 "\tAny birthday in coming N days (including today) will be "
	echo -e	 "\t\t\tnotified"
	echo -ne "\t-D, --debug"
	echo -e  "\tTurn debugging on. Will display debugging messages"
	echo -ne "\t-h, --help"
	echo -e  "\tDisplay this help message and exit"
}

#
# Name:		trim
# Inputs:	A string
# Output:	String with leading and trailing white spaces removed
#
function trim {
	string="$1"
	string=`echo "$1" | sed s/'^ *'// | sed s/' *$'//`
	echo "$string"
}


### Main program ###

DATA_FILE="$PROG_PATH"/bday.dat	# Name of the text file that contains list 
								# of all birthdays

PRIOR_DAYS=7		# How many days in advance should it start
					# notifying you; default is 7

DEBUG=0				# Print debugging info if set to 1


# Parse command line arguments and set appropriate options
while [ "$#" -gt 0 ]
do
	case "$1" in
		'' | ' ')
			;;
		-d | --days)
			shift
			PRIOR_DAYS="$1"
			;;
		-D | --debug)
			DEBUG=1
			;;
		-h | --help)
			display_usage
			exit 0
			;;
		*)
			echo "Unrecognized option specified!" 1>&2
			echo "Exiting ..." 1>&2
			exit 1
			;;
	esac
	shift
done

# Generate the dates for the next $PRIOR_DAYS days including today
next_dates=
for (( i = 0; i < $PRIOR_DAYS; i++ ))
do
	next_dates[$i]=`date --date "$i day" '+%b-%d'`
done

if [ "$DEBUG" -eq 1 ]
then
	echo "Today: `date`"
	echo "Program path: $PROG_PATH"
	echo "Database: $DATA_FILE"
	echo "Days prior to notification: $PRIOR_DAYS"
	echo "Debug: $DEBUG"
	echo -n "Date range: ${next_dates[0]} to "
	echo "${next_dates[(($PRIOR_DAYS - 1))]}"
	echo ""
fi

names=
dates=
mesgs=
n_records=0
i=0

# Read the birthday database and load the names and dates 
# in two different arrays
while read a_record
do
	# Skip commented lines
	a_record=`trim "$a_record"`

	echo "$a_record" | grep '^[[:space:]]*#' | grep -v 'grep' 1> /dev/null 2>&1
	if [ $? -eq 0 ]
	then
		#echo 'Comment'
		continue
	fi

	name=`echo "$a_record" | cut -d'|' -f1`
	bday=`echo "$a_record" | cut -d'|' -f2`
	mesg=`echo "$a_record" | cut -d'|' -f3`
	name=`trim "$name"`
	bday=`trim "$bday"`
	mesg=`trim "$mesg"`

	names[$i]=$name
	dates[$i]=$bday
	mesgs[$i]=$mesg
	n_records=`expr $n_records + 1`
	i=`expr $i + 1`
done < "$DATA_FILE"

if [ "$DEBUG" -eq 1 ]
then
	echo "Currently active records from the database:"
	for (( i = 0; i < ${#names[@]}; i++ ))
	do
		echo "$i: ${names[$i]}, ${dates[$i]}"
	done
fi

# Now the important part
# For each of the date found in the database
# Check if the date lies in the next 7 days (or whatever specified)
echo "Upcoming birthdays in next $PRIOR_DAYS days:"
today=`date '+%b-%d'`
tomorrow=$(date '+%b-%d' --date '1 day')

for (( i = 0; i < $n_records; i++ ))
do
	for (( j = 0; j < $PRIOR_DAYS; j++ ))
	do
		if [ "${dates[$i]}" = "${next_dates[$j]}" ]
		then
			if [ "$today" = "${dates[$i]}" ]
			then
				status='[Today]'
			elif [ "$tomorrow" = "${dates[$i]}" ]
			then
				status='[Tomorrow]'
			else
				status=''
			fi

			echo "<${mesgs[$i]}> ${names[$i]}, ${dates[$i]} $status"
			break
		fi
	done
done

exit 0
